@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Scalac.scala 10987 2007-05-10 13:38:25Z dubochet $
+// $Id: Scalac.scala 12695 2007-08-27 10:14:49Z michelou $
 
 package scala.tools.ant
 
@@ -49,7 +49,8 @@ import scala.tools.nsc.reporters.{Reporter, ConsoleReporter}
  *    <li>addparams,</li>
  *    <li>scalacdebugging,</li>
  *    <li>deprecation,</li>
- *    <li>unchecked.</li>
+ *    <li>unchecked,</li>
+ *    <li>failonerror.</li>
  *  </ul>
  *  <p>
  *    It also takes the following parameters as nested elements:
@@ -61,7 +62,9 @@ import scala.tools.nsc.reporters.{Reporter, ConsoleReporter}
  *    <li>bootclasspath,</li>
  *    <li>extdirs.</li>
  * </ul>
- * @author Gilles Dubochet, Stephane Micheloud */
+ *
+ * @author Gilles Dubochet, Stephane Micheloud
+ */
 class Scalac extends MatchingTask {
 
   /** The unique Ant file utilities instance to use in this task. */
@@ -74,7 +77,7 @@ class Scalac extends MatchingTask {
   abstract class PermissibleValue {
     val values: List[String]
     def isPermissible(value: String): Boolean =
-      (value == "") || values.exists(.startsWith(value))
+      (value == "") || values.exists(_.startsWith(value))
   }
 
   /** Defines valid values for the logging property. */
@@ -139,6 +142,8 @@ class Scalac extends MatchingTask {
   private var deprecation: Option[Boolean] = None
   /** Instruct the compiler to generate unchecked information. */
   private var unchecked: Option[Boolean] = None
+  /** Indicates whether compilation errors will fail the build; defaults to true. */
+  private var failonerror: Boolean = true
 
   // Name of the output assembly (only relevant with -target:msil)
   private var assemname: Option[String] = None
@@ -149,15 +154,18 @@ class Scalac extends MatchingTask {
    *  in case of failure. */
   private var scalacDebugging: Boolean = false
 
+  private var generics: Option[Boolean] = None
+
 /*============================================================================*\
 **                             Properties setters                             **
 \*============================================================================*/
 
   /** Sets the srcdir attribute. Used by Ant.
    *  @param input The value of <code>origin</code>. */
-  def setSrcdir(input: Path) =
+  def setSrcdir(input: Path) {
     if (origin.isEmpty) origin = Some(input)
     else origin.get.append(input)
+  }
 
   /** Sets the <code>origin</code> as a nested src Ant parameter.
    *  @return An origin path to be configured. */
@@ -173,14 +181,14 @@ class Scalac extends MatchingTask {
 
   /** Sets the <code>destdir</code> attribute. Used by Ant.
    *  @param input The value of <code>destination</code>. */
-  def setDestdir(input: File) =
-      destination = Some(input)
+  def setDestdir(input: File) { destination = Some(input) }
 
   /** Sets the <code>classpath</code> attribute. Used by Ant.
    *  @param input The value of <code>classpath</code>. */
-  def setClasspath(input: Path) =
+  def setClasspath(input: Path) {
     if (classpath.isEmpty) classpath = Some(input)
     else classpath.get.append(input)
+  }
 
   /** Sets the <code>classpath</code> as a nested classpath Ant parameter.
    *  @return A class path to be configured. */
@@ -191,14 +199,16 @@ class Scalac extends MatchingTask {
 
   /** Sets the <code>classpath</code> as an external reference Ant parameter.
    *  @param input A reference to a class path. */
-  def setClasspathref(input: Reference) =
+  def setClasspathref(input: Reference) {
     createClasspath().setRefid(input)
+  }
 
   /** Sets the <code>sourcepath</code> attribute. Used by Ant.
    *  @param input The value of <code>sourcepath</code>. */
-  def setSourcepath(input: Path) =
+  def setSourcepath(input: Path) {
     if (sourcepath.isEmpty) sourcepath = Some(input)
     else sourcepath.get.append(input)
+  }
 
   /** Sets the <code>sourcepath</code> as a nested sourcepath Ant parameter.
    *  @return A source path to be configured. */
@@ -209,15 +219,17 @@ class Scalac extends MatchingTask {
 
   /** Sets the <code>sourcepath</code> as an external reference Ant parameter.
    *  @param input A reference to a source path. */
-  def setSourcepathref(input: Reference) =
+  def setSourcepathref(input: Reference) {
     createSourcepath().setRefid(input)
+  }
 
   /** Sets the boot classpath attribute. Used by Ant.
    *
    *  @param input The value of <code>bootclasspath</code>. */
-  def setBootclasspath(input: Path) =
+  def setBootclasspath(input: Path) {
     if (bootclasspath.isEmpty) bootclasspath = Some(input)
     else bootclasspath.get.append(input)
+  }
 
   /** Sets the <code>bootclasspath</code> as a nested sourcepath Ant
    *  parameter.
@@ -264,18 +276,18 @@ class Scalac extends MatchingTask {
 
   /** Sets the <code>force</code> attribute. Used by Ant.
    *  @param input The value for <code>force</code>. */
-  def setForce(input: Boolean): Unit =
-    force = input
+  def setForce(input: Boolean) { force = input }
 
   /** Sets the logging level attribute. Used by Ant.
    *  @param input The value for <code>logging</code>. */
-  def setLogging(input: String) =
+  def setLogging(input: String) {
     if (LoggingLevel.isPermissible(input)) logging = Some(input)
     else error("Logging level '" + input + "' does not exist.")
+  }
 
   /** Sets the <code>logphase</code> attribute. Used by Ant.
    *  @param input The value for <code>logPhase</code>. */
-  def setLogPhase(input: String) = {
+  def setLogPhase(input: String) {
     logPhase = List.fromArray(input.split(",")).flatMap { s: String =>
       val st = s.trim()
       if (CompilerPhase.isPermissible(st))
@@ -289,45 +301,47 @@ class Scalac extends MatchingTask {
 
   /** Sets the <code>usepredefs</code> attribute. Used by Ant.
    *  @param input The value for <code>usepredefs</code>. */
-  def setUsepredefs(input: Boolean): Unit =
-    usepredefs = Some(input)
+  def setUsepredefs(input: Boolean) { usepredefs = Some(input) }
 
   /** Set the <code>debug</code> info attribute.
    *  @param input The value for <code>debug</code>. */
-  def setDebuginfo(input: String): Unit =
-    debugInfo = Some(input)
+  def setDebuginfo(input: String) { debugInfo = Some(input) }
 
   /** Set the <code>addparams</code> info attribute.
    *  @param input The value for <code>addparams</code>. */
-  def setAddparams(input: String): Unit =
-    addParams = input
+  def setAddparams(input: String) { addParams = input }
 
   /** Set the <code>deprecation</code> info attribute.
    *  @param input One of the flags <code>yes/no</code> or <code>on/off</code>. */
-  def setDeprecation(input: String): Unit =
+  def setDeprecation(input: String) {
     if (Flag.isPermissible(input))
       deprecation = Some("yes" == input || "on" == input)
     else
       error("Unknown deprecation flag '" + input + "'")
+  }
 
   /** Set the <code>unchecked</code> info attribute.
    *  @param input One of the flags <code>yes/no</code> or <code>on/off</code>. */
-  def setUnchecked(input: String): Unit =
+  def setUnchecked(input: String) {
     if (Flag.isPermissible(input))
       unchecked = Some("yes" == input || "on" == input)
     else
       error("Unknown unchecked flag '" + input + "'")
+  }
+
+  /** Sets the <code>force</code> attribute. Used by Ant.
+   *  @param input The value for <code>force</code>. */
+  def setFailonerror(input: Boolean) { failonerror = input }
 
   /** Set the <code>scalacdebugging</code> info attribute.
    *  @param input The specified flag */
-  def setScalacdebugging(input: Boolean): Unit =
-    scalacDebugging = input
+  def setScalacdebugging(input: Boolean) { scalacDebugging = input }
 
-  def setAssemname(input: String): Unit =
-    assemname = Some(input)
+  def setAssemname(input: String) { assemname = Some(input) }
 
-  def setAssemrefs(input: String): Unit =
-    assemrefs = Some(input)
+  def setAssemrefs(input: String) { assemrefs = Some(input) }
+
+  def setGenerics(input: Boolean) { generics = Some(input) }
 
 /*============================================================================*\
 **                             Properties getters                             **
@@ -352,7 +366,7 @@ class Scalac extends MatchingTask {
    *  @return The destination as a file. */
   private def getDestination: File =
     if (destination.isEmpty) error("Member 'destination' is empty.")
-    else existing(getProject().resolveFile(destination.get.toString()))
+    else existing(getProject().resolveFile(destination.get.toString))
 
   /** Gets the value of the <code>sourcepath</code> attribute in a
    *  Scala-friendly form.
@@ -406,7 +420,7 @@ class Scalac extends MatchingTask {
    *  @return     The same file. */
   private def existing(file: File): File = {
     if (!file.exists())
-      log("Element '" + file.toString() + "' does not exist.",
+      log("Element '" + file.toString + "' does not exist.",
           Project.MSG_WARN)
     file
   }
@@ -430,12 +444,24 @@ class Scalac extends MatchingTask {
   private def error(message: String): Nothing =
     throw new BuildException(message, getLocation())
 
+
+/*============================================================================*\
+**                      Hooks for variants of Scala                           **
+\*============================================================================*/
+
+  protected val sourceEnding = ".scala"
+  protected def newSettings(error: String=>Unit): Settings =
+    new Settings(error)
+  protected def newGlobal(settings: Settings, reporter: Reporter) =
+    new Global(settings, reporter)
+
+
 /*============================================================================*\
 **                           The big execute method                           **
 \*============================================================================*/
 
   /** Initializes settings and source files */
-  protected def initialize: Pair[Settings, List[File]] = {
+  protected def initialize: (Settings, List[File]) = {
     // Tests if all mandatory attributes are set and valid.
     if (origin.isEmpty) error("Attribute 'srcdir' is not set.")
     if (getOrigin.isEmpty) error("Attribute 'srcdir' is not set.")
@@ -445,7 +471,7 @@ class Scalac extends MatchingTask {
 
     val mapper = new GlobPatternMapper()
     mapper.setTo("*.class")
-    mapper.setFrom("*.scala")
+    mapper.setFrom("*" + sourceEnding)
 
     // Scans source directories to build up a compile lists.
     // If force is false, only files were the .class file in destination is
@@ -468,13 +494,13 @@ class Scalac extends MatchingTask {
                 (if (list.length > 1) "s: " else ": "),
                 ", ",
                 " "
-              ) + "to " + getDestination.toString()
+              ) + "to " + getDestination.toString
             )
           else if (list.length > 0)
             log(
               "Compiling " + list.length + " source file" +
               (if (list.length > 1) "s" else "") +
-              (" to " + getDestination.toString())
+              (" to " + getDestination.toString)
             )
           else
             log("No files selected for compilation", Project.MSG_VERBOSE)
@@ -483,13 +509,13 @@ class Scalac extends MatchingTask {
         }
       }
       yield {
-        log(originFile.toString(), Project.MSG_DEBUG)
+        log(originFile.toString, Project.MSG_DEBUG)
         nameToFile(originDir)(originFile)
       }
 
     // Builds-up the compilation settings for Scalac with the existing Ant
     // parameters.
-    val settings = new Settings(error)
+    val settings = newSettings(error)
     settings.outdir.value = asString(destination.get)
     if (!classpath.isEmpty)
       settings.classpath.value = asString(getClasspath)
@@ -516,11 +542,12 @@ class Scalac extends MatchingTask {
 
     if (!assemname.isEmpty) settings.assemname.value = assemname.get
     if (!assemrefs.isEmpty) settings.assemrefs.value = assemrefs.get
+    if (!generics.isEmpty) settings.Xgenerics.value = generics.get
 
     log("Scalac params = '" + addParams + "'", Project.MSG_DEBUG)
     var args =
       if (addParams.trim() == "") Nil
-      else List.fromArray(addParams.trim().split(" ")).map(.trim())
+      else List.fromArray(addParams.trim().split(" ")).map(_.trim())
     while (!args.isEmpty) {
       val argsBuf = args
       if (args.head startsWith "-") {
@@ -531,18 +558,18 @@ class Scalac extends MatchingTask {
       if (argsBuf eq args)
         error("Parameter '" + args.head + "' is not recognised by Scalac.")
     }
-    Pair(settings, sourceFiles)
+    (settings, sourceFiles)
   }
 
   /** Performs the compilation. */
   override def execute() = {
-    val Pair(settings, sourceFiles) = initialize
+    val (settings, sourceFiles) = initialize
     val reporter = new ConsoleReporter(settings)
 
     // Compiles the actual code
-    val compiler = new Global(settings, reporter)
+    val compiler = newGlobal(settings, reporter)
     try {
-      (new compiler.Run).compile(sourceFiles.map (.toString()))
+      (new compiler.Run).compile(sourceFiles.map (_.toString))
      }
      catch {
       case exception: Throwable if (exception.getMessage ne null) =>
@@ -555,12 +582,14 @@ class Scalac extends MatchingTask {
               "(no error message provided); see the error output for details.")
     }
     reporter.printSummary()
-    if (reporter.hasErrors)
-      error(
-          "Compile failed with " +
-          reporter.ERROR.count + " error" +
-          (if (reporter.ERROR.count > 1) "s" else "") +
-          "; see the compiler error output for details.")
+    if (reporter.hasErrors) {
+      val msg =
+        "Compile failed with " +
+        reporter.ERROR.count + " error" +
+        (if (reporter.ERROR.count > 1) "s" else "") +
+        "; see the compiler error output for details."
+      if (failonerror) error(msg) else log(msg)
+    }
     else if (reporter.WARNING.count > 0)
       log(
           "Compile suceeded with " +
